# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import AccountcommonTagResponse
from ..models import AccountcommonTagsGetResponseV3
from ..models import ModelTagCreateRequestV3
from ..models import ModelTagUpdateRequestV3
from ..models import RestErrorResponse

from ..operations.account_identifier_tag import AdminCreateTagV3
from ..operations.account_identifier_tag import AdminDeleteTagV3
from ..operations.account_identifier_tag import AdminQueryTagV3
from ..operations.account_identifier_tag import AdminUpdateTagV3


@same_doc_as(AdminCreateTagV3)
def admin_create_tag_v3(
    body: ModelTagCreateRequestV3,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create Account Identifier Tag (AdminCreateTagV3)

    Creates a new Account Identifier Tag for users, which can be used to identify and categorize user accounts.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/tags

        method: POST

        tags: ["Account Identifier Tag"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelTagCreateRequestV3 in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - AccountcommonTagResponse (Created)

        400: Bad Request - RestErrorResponse (20019: unable to parse request body | 20002: validation error)

        409: Conflict - RestErrorResponse

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminCreateTagV3.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminCreateTagV3)
async def admin_create_tag_v3_async(
    body: ModelTagCreateRequestV3,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create Account Identifier Tag (AdminCreateTagV3)

    Creates a new Account Identifier Tag for users, which can be used to identify and categorize user accounts.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/tags

        method: POST

        tags: ["Account Identifier Tag"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelTagCreateRequestV3 in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - AccountcommonTagResponse (Created)

        400: Bad Request - RestErrorResponse (20019: unable to parse request body | 20002: validation error)

        409: Conflict - RestErrorResponse

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminCreateTagV3.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminDeleteTagV3)
def admin_delete_tag_v3(
    tag_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete Account Identifier Tag (AdminDeleteTagV3)

    Deletes an Account Identifier Tag used to identify and categorize user accounts.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/tags/{tagId}

        method: DELETE

        tags: ["Account Identifier Tag"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        tag_id: (tagId) REQUIRED str in path

    Responses:
        204: No Content - (No Content)

        404: Not Found - RestErrorResponse

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminDeleteTagV3.create(
        tag_id=tag_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminDeleteTagV3)
async def admin_delete_tag_v3_async(
    tag_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete Account Identifier Tag (AdminDeleteTagV3)

    Deletes an Account Identifier Tag used to identify and categorize user accounts.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/tags/{tagId}

        method: DELETE

        tags: ["Account Identifier Tag"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        tag_id: (tagId) REQUIRED str in path

    Responses:
        204: No Content - (No Content)

        404: Not Found - RestErrorResponse

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminDeleteTagV3.create(
        tag_id=tag_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminQueryTagV3)
def admin_query_tag_v3(
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    tag_name: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Query Account Identifier Tag (AdminQueryTagV3)

    Retrieves Account Identifier Tags used to identify and categorize user accounts.
    Tag Name can be used for partial content search.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/tags

        method: GET

        tags: ["Account Identifier Tag"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        tag_name: (tagName) OPTIONAL str in query

    Responses:
        200: OK - AccountcommonTagsGetResponseV3 (OK)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminQueryTagV3.create(
        limit=limit,
        offset=offset,
        tag_name=tag_name,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminQueryTagV3)
async def admin_query_tag_v3_async(
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    tag_name: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Query Account Identifier Tag (AdminQueryTagV3)

    Retrieves Account Identifier Tags used to identify and categorize user accounts.
    Tag Name can be used for partial content search.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/tags

        method: GET

        tags: ["Account Identifier Tag"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        tag_name: (tagName) OPTIONAL str in query

    Responses:
        200: OK - AccountcommonTagsGetResponseV3 (OK)

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminQueryTagV3.create(
        limit=limit,
        offset=offset,
        tag_name=tag_name,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminUpdateTagV3)
def admin_update_tag_v3(
    body: ModelTagUpdateRequestV3,
    tag_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update Account Identifier Tag (AdminUpdateTagV3)

    Updates the details of an existing Account Identifier Tag used to identify and categorize user accounts.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/tags/{tagId}

        method: PUT

        tags: ["Account Identifier Tag"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelTagUpdateRequestV3 in body

        namespace: (namespace) REQUIRED str in path

        tag_id: (tagId) REQUIRED str in path

    Responses:
        200: OK - AccountcommonTagResponse (OK)

        400: Bad Request - RestErrorResponse (20019: unable to parse request body | 20002: validation error)

        404: Not Found - RestErrorResponse

        409: Conflict - RestErrorResponse

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminUpdateTagV3.create(
        body=body,
        tag_id=tag_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminUpdateTagV3)
async def admin_update_tag_v3_async(
    body: ModelTagUpdateRequestV3,
    tag_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update Account Identifier Tag (AdminUpdateTagV3)

    Updates the details of an existing Account Identifier Tag used to identify and categorize user accounts.

    Properties:
        url: /iam/v3/admin/namespaces/{namespace}/tags/{tagId}

        method: PUT

        tags: ["Account Identifier Tag"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelTagUpdateRequestV3 in body

        namespace: (namespace) REQUIRED str in path

        tag_id: (tagId) REQUIRED str in path

    Responses:
        200: OK - AccountcommonTagResponse (OK)

        400: Bad Request - RestErrorResponse (20019: unable to parse request body | 20002: validation error)

        404: Not Found - RestErrorResponse

        409: Conflict - RestErrorResponse

        500: Internal Server Error - RestErrorResponse (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminUpdateTagV3.create(
        body=body,
        tag_id=tag_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the AccelByte Gaming Services Login Queue Service."""

__version__ = "2.5.1"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

# admin_v1
from .wrappers import admin_get_configuration
from .wrappers import admin_get_configuration_async
from .wrappers import admin_get_status
from .wrappers import admin_get_status_async
from .wrappers import admin_update_configuration
from .wrappers import admin_update_configuration_async

# ticket_v1
from .wrappers import cancel_ticket
from .wrappers import cancel_ticket_async
from .wrappers import refresh_ticket
from .wrappers import refresh_ticket_async



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the AccelByte Gaming Services Platform Service."""

__version__ = "6.14.0"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .commit import Commit
from .get_trade_history_by_criteria import GetTradeHistoryByCriteria
from .get_trade_history_by_criteria import (
    StatusEnum as GetTradeHistoryByCriteriaStatusEnum,
)
from .get_trade_history_by_tr_8b774c import GetTradeHistoryByTransactionId



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

# AccelByte Gaming Services Platform Service

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from .....core import Operation
from .....core import HeaderStr
from .....core import HttpResponse
from .....core import StrEnum

from ...models import ErrorEntity


class FeaturesToCheckEnum(StrEnum):
    CAMPAIGN = "CAMPAIGN"
    CATALOG = "CATALOG"
    DLC = "DLC"
    ENTITLEMENT = "ENTITLEMENT"
    IAP = "IAP"
    REWARD = "REWARD"


class DeleteItem(Operation):
    """Delete an item (deleteItem)

    This API is used to delete an item permanently.

    force: the default value should be: false. When the value is:
    * false: only the items in the draft store that have never been published yet can be removed.
    *  true: the item in the draft store(even been published before) can be removed.

    Properties:
        url: /platform/admin/namespaces/{namespace}/items/{itemId}

        method: DELETE

        tags: ["Item"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        item_id: (itemId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        features_to_check: (featuresToCheck) OPTIONAL List[Union[str, FeaturesToCheckEnum]] in query

        force: (force) OPTIONAL bool in query

        store_id: (storeId) OPTIONAL str in query

    Responses:
        204: No Content - (Delete item successfully)

        404: Not Found - ErrorEntity (30141: Store [{storeId}] does not exist in namespace [{namespace}] | 30142: Published store does not exist in namespace [{namespace}] | 30341: Item [{itemId}] does not exist in namespace [{namespace}] | 30335: Item [{itemId}] can't be deleted in non-forced mode if item has been published)

        409: Conflict - ErrorEntity (30386: The item [{itemId}] is currently associated and cannot be deleted in namespace [{namespace}], Feature {featureName}, Module {moduleName}, and Reference ID {referenceId} are using this item ID)
    """

    # region fields

    _url: str = "/platform/admin/namespaces/{namespace}/items/{itemId}"
    _method: str = "DELETE"
    _consumes: List[str] = []
    _produces: List[str] = ["application/json"]
    _securities: List[List[str]] = [["BEARER_AUTH"]]
    _location_query: str = None

    item_id: str  # REQUIRED in [path]
    namespace: str  # REQUIRED in [path]
    features_to_check: List[Union[str, FeaturesToCheckEnum]]  # OPTIONAL in [query]
    force: bool  # OPTIONAL in [query]
    store_id: str  # OPTIONAL in [query]

    # endregion fields

    # region properties

    @property
    def url(self) -> str:
        return self._url

    @property
    def method(self) -> str:
        return self._method

    @property
    def consumes(self) -> List[str]:
        return self._consumes

    @property
    def produces(self) -> List[str]:
        return self._produces

    @property
    def securities(self) -> List[List[str]]:
        return self._securities

    @property
    def location_query(self) -> str:
        return self._location_query

    # endregion properties

    # region get methods

    # endregion get methods

    # region get_x_params methods

    def get_all_params(self) -> dict:
        return {
            "path": self.get_path_params(),
            "query": self.get_query_params(),
        }

    def get_path_params(self) -> dict:
        result = {}
        if hasattr(self, "item_id"):
            result["itemId"] = self.item_id
        if hasattr(self, "namespace"):
            result["namespace"] = self.namespace
        return result

    def get_query_params(self) -> dict:
        result = {}
        if hasattr(self, "features_to_check"):
            result["featuresToCheck"] = self.features_to_check
        if hasattr(self, "force"):
            result["force"] = self.force
        if hasattr(self, "store_id"):
            result["storeId"] = self.store_id
        return result

    # endregion get_x_params methods

    # region is/has methods

    # endregion is/has methods

    # region with_x methods

    def with_item_id(self, value: str) -> DeleteItem:
        self.item_id = value
        return self

    def with_namespace(self, value: str) -> DeleteItem:
        self.namespace = value
        return self

    def with_features_to_check(
        self, value: List[Union[str, FeaturesToCheckEnum]]
    ) -> DeleteItem:
        self.features_to_check = value
        return self

    def with_force(self, value: bool) -> DeleteItem:
        self.force = value
        return self

    def with_store_id(self, value: str) -> DeleteItem:
        self.store_id = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "item_id") and self.item_id:
            result["itemId"] = str(self.item_id)
        elif include_empty:
            result["itemId"] = ""
        if hasattr(self, "namespace") and self.namespace:
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "features_to_check") and self.features_to_check:
            result["featuresToCheck"] = [str(i0) for i0 in self.features_to_check]
        elif include_empty:
            result["featuresToCheck"] = []
        if hasattr(self, "force") and self.force:
            result["force"] = bool(self.force)
        elif include_empty:
            result["force"] = False
        if hasattr(self, "store_id") and self.store_id:
            result["storeId"] = str(self.store_id)
        elif include_empty:
            result["storeId"] = ""
        return result

    # endregion to methods

    # region response methods

    # noinspection PyMethodMayBeStatic
    def parse_response(
        self, code: int, content_type: str, content: Any
    ) -> Tuple[None, Union[None, ErrorEntity, HttpResponse]]:
        """Parse the given response.

        204: No Content - (Delete item successfully)

        404: Not Found - ErrorEntity (30141: Store [{storeId}] does not exist in namespace [{namespace}] | 30142: Published store does not exist in namespace [{namespace}] | 30341: Item [{itemId}] does not exist in namespace [{namespace}] | 30335: Item [{itemId}] can't be deleted in non-forced mode if item has been published)

        409: Conflict - ErrorEntity (30386: The item [{itemId}] is currently associated and cannot be deleted in namespace [{namespace}], Feature {featureName}, Module {moduleName}, and Reference ID {referenceId} are using this item ID)

        ---: HttpResponse (Undocumented Response)

        ---: HttpResponse (Unexpected Content-Type Error)

        ---: HttpResponse (Unhandled Error)
        """
        pre_processed_response, error = self.pre_process_response(
            code=code, content_type=content_type, content=content
        )
        if error is not None:
            return None, None if error.is_no_content() else error
        code, content_type, content = pre_processed_response

        if code == 204:
            return None, None
        if code == 404:
            return None, ErrorEntity.create_from_dict(content)
        if code == 409:
            return None, ErrorEntity.create_from_dict(content)

        return self.handle_undocumented_response(
            code=code, content_type=content_type, content=content
        )

    # endregion response methods

    # region static methods

    @classmethod
    def create(
        cls,
        item_id: str,
        namespace: str,
        features_to_check: Optional[List[Union[str, FeaturesToCheckEnum]]] = None,
        force: Optional[bool] = None,
        store_id: Optional[str] = None,
        **kwargs,
    ) -> DeleteItem:
        instance = cls()
        instance.item_id = item_id
        instance.namespace = namespace
        if features_to_check is not None:
            instance.features_to_check = features_to_check
        if force is not None:
            instance.force = force
        if store_id is not None:
            instance.store_id = store_id
        if x_flight_id := kwargs.get("x_flight_id", None):
            instance.x_flight_id = x_flight_id
        return instance

    @classmethod
    def create_from_dict(cls, dict_: dict, include_empty: bool = False) -> DeleteItem:
        instance = cls()
        if "itemId" in dict_ and dict_["itemId"] is not None:
            instance.item_id = str(dict_["itemId"])
        elif include_empty:
            instance.item_id = ""
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "featuresToCheck" in dict_ and dict_["featuresToCheck"] is not None:
            instance.features_to_check = [str(i0) for i0 in dict_["featuresToCheck"]]
        elif include_empty:
            instance.features_to_check = []
        if "force" in dict_ and dict_["force"] is not None:
            instance.force = bool(dict_["force"])
        elif include_empty:
            instance.force = False
        if "storeId" in dict_ and dict_["storeId"] is not None:
            instance.store_id = str(dict_["storeId"])
        elif include_empty:
            instance.store_id = ""
        return instance

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "itemId": "item_id",
            "namespace": "namespace",
            "featuresToCheck": "features_to_check",
            "force": "force",
            "storeId": "store_id",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "itemId": True,
            "namespace": True,
            "featuresToCheck": False,
            "force": False,
            "storeId": False,
        }

    @staticmethod
    def get_collection_format_map() -> Dict[str, Union[None, str]]:
        return {
            "featuresToCheck": "multi",  # in query
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "featuresToCheck": [
                "CAMPAIGN",
                "CATALOG",
                "DLC",
                "ENTITLEMENT",
                "IAP",
                "REWARD",
            ],  # in query
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Inventory Service Early Access

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model
from ....core import StrEnum


class SourceEnum(StrEnum):
    ECOMMERCE = "ECOMMERCE"
    OTHER = "OTHER"


class ApimodelsSaveItemReq(Model):
    """Apimodels save item req (apimodels.SaveItemReq)

    Properties:
        custom_attributes: (customAttributes) REQUIRED Dict[str, Any]

        inventory_configuration_code: (inventoryConfigurationCode) REQUIRED str

        qty: (qty) REQUIRED int

        server_custom_attributes: (serverCustomAttributes) REQUIRED Dict[str, Any]

        slot_id: (slotId) REQUIRED str

        slot_used: (slotUsed) REQUIRED int

        source: (source) REQUIRED Union[str, SourceEnum]

        source_item_id: (sourceItemId) REQUIRED str

        tags: (tags) REQUIRED List[str]

        type_: (type) REQUIRED str
    """

    # region fields

    custom_attributes: Dict[str, Any]  # REQUIRED
    inventory_configuration_code: str  # REQUIRED
    qty: int  # REQUIRED
    server_custom_attributes: Dict[str, Any]  # REQUIRED
    slot_id: str  # REQUIRED
    slot_used: int  # REQUIRED
    source: Union[str, SourceEnum]  # REQUIRED
    source_item_id: str  # REQUIRED
    tags: List[str]  # REQUIRED
    type_: str  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_custom_attributes(self, value: Dict[str, Any]) -> ApimodelsSaveItemReq:
        self.custom_attributes = value
        return self

    def with_inventory_configuration_code(self, value: str) -> ApimodelsSaveItemReq:
        self.inventory_configuration_code = value
        return self

    def with_qty(self, value: int) -> ApimodelsSaveItemReq:
        self.qty = value
        return self

    def with_server_custom_attributes(
        self, value: Dict[str, Any]
    ) -> ApimodelsSaveItemReq:
        self.server_custom_attributes = value
        return self

    def with_slot_id(self, value: str) -> ApimodelsSaveItemReq:
        self.slot_id = value
        return self

    def with_slot_used(self, value: int) -> ApimodelsSaveItemReq:
        self.slot_used = value
        return self

    def with_source(self, value: Union[str, SourceEnum]) -> ApimodelsSaveItemReq:
        self.source = value
        return self

    def with_source_item_id(self, value: str) -> ApimodelsSaveItemReq:
        self.source_item_id = value
        return self

    def with_tags(self, value: List[str]) -> ApimodelsSaveItemReq:
        self.tags = value
        return self

    def with_type(self, value: str) -> ApimodelsSaveItemReq:
        self.type_ = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "custom_attributes"):
            result["customAttributes"] = {
                str(k0): v0 for k0, v0 in self.custom_attributes.items()
            }
        elif include_empty:
            result["customAttributes"] = {}
        if hasattr(self, "inventory_configuration_code"):
            result["inventoryConfigurationCode"] = str(
                self.inventory_configuration_code
            )
        elif include_empty:
            result["inventoryConfigurationCode"] = ""
        if hasattr(self, "qty"):
            result["qty"] = int(self.qty)
        elif include_empty:
            result["qty"] = 0
        if hasattr(self, "server_custom_attributes"):
            result["serverCustomAttributes"] = {
                str(k0): v0 for k0, v0 in self.server_custom_attributes.items()
            }
        elif include_empty:
            result["serverCustomAttributes"] = {}
        if hasattr(self, "slot_id"):
            result["slotId"] = str(self.slot_id)
        elif include_empty:
            result["slotId"] = ""
        if hasattr(self, "slot_used"):
            result["slotUsed"] = int(self.slot_used)
        elif include_empty:
            result["slotUsed"] = 0
        if hasattr(self, "source"):
            result["source"] = str(self.source)
        elif include_empty:
            result["source"] = Union[str, SourceEnum]()
        if hasattr(self, "source_item_id"):
            result["sourceItemId"] = str(self.source_item_id)
        elif include_empty:
            result["sourceItemId"] = ""
        if hasattr(self, "tags"):
            result["tags"] = [str(i0) for i0 in self.tags]
        elif include_empty:
            result["tags"] = []
        if hasattr(self, "type_"):
            result["type"] = str(self.type_)
        elif include_empty:
            result["type"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        custom_attributes: Dict[str, Any],
        inventory_configuration_code: str,
        qty: int,
        server_custom_attributes: Dict[str, Any],
        slot_id: str,
        slot_used: int,
        source: Union[str, SourceEnum],
        source_item_id: str,
        tags: List[str],
        type_: str,
        **kwargs,
    ) -> ApimodelsSaveItemReq:
        instance = cls()
        instance.custom_attributes = custom_attributes
        instance.inventory_configuration_code = inventory_configuration_code
        instance.qty = qty
        instance.server_custom_attributes = server_custom_attributes
        instance.slot_id = slot_id
        instance.slot_used = slot_used
        instance.source = source
        instance.source_item_id = source_item_id
        instance.tags = tags
        instance.type_ = type_
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ApimodelsSaveItemReq:
        instance = cls()
        if not dict_:
            return instance
        if "customAttributes" in dict_ and dict_["customAttributes"] is not None:
            instance.custom_attributes = {
                str(k0): v0 for k0, v0 in dict_["customAttributes"].items()
            }
        elif include_empty:
            instance.custom_attributes = {}
        if (
            "inventoryConfigurationCode" in dict_
            and dict_["inventoryConfigurationCode"] is not None
        ):
            instance.inventory_configuration_code = str(
                dict_["inventoryConfigurationCode"]
            )
        elif include_empty:
            instance.inventory_configuration_code = ""
        if "qty" in dict_ and dict_["qty"] is not None:
            instance.qty = int(dict_["qty"])
        elif include_empty:
            instance.qty = 0
        if (
            "serverCustomAttributes" in dict_
            and dict_["serverCustomAttributes"] is not None
        ):
            instance.server_custom_attributes = {
                str(k0): v0 for k0, v0 in dict_["serverCustomAttributes"].items()
            }
        elif include_empty:
            instance.server_custom_attributes = {}
        if "slotId" in dict_ and dict_["slotId"] is not None:
            instance.slot_id = str(dict_["slotId"])
        elif include_empty:
            instance.slot_id = ""
        if "slotUsed" in dict_ and dict_["slotUsed"] is not None:
            instance.slot_used = int(dict_["slotUsed"])
        elif include_empty:
            instance.slot_used = 0
        if "source" in dict_ and dict_["source"] is not None:
            instance.source = str(dict_["source"])
        elif include_empty:
            instance.source = Union[str, SourceEnum]()
        if "sourceItemId" in dict_ and dict_["sourceItemId"] is not None:
            instance.source_item_id = str(dict_["sourceItemId"])
        elif include_empty:
            instance.source_item_id = ""
        if "tags" in dict_ and dict_["tags"] is not None:
            instance.tags = [str(i0) for i0 in dict_["tags"]]
        elif include_empty:
            instance.tags = []
        if "type" in dict_ and dict_["type"] is not None:
            instance.type_ = str(dict_["type"])
        elif include_empty:
            instance.type_ = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ApimodelsSaveItemReq]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ApimodelsSaveItemReq]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ApimodelsSaveItemReq,
        List[ApimodelsSaveItemReq],
        Dict[Any, ApimodelsSaveItemReq],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "customAttributes": "custom_attributes",
            "inventoryConfigurationCode": "inventory_configuration_code",
            "qty": "qty",
            "serverCustomAttributes": "server_custom_attributes",
            "slotId": "slot_id",
            "slotUsed": "slot_used",
            "source": "source",
            "sourceItemId": "source_item_id",
            "tags": "tags",
            "type": "type_",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "customAttributes": True,
            "inventoryConfigurationCode": True,
            "qty": True,
            "serverCustomAttributes": True,
            "slotId": True,
            "slotUsed": True,
            "source": True,
            "sourceItemId": True,
            "tags": True,
            "type": True,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "source": ["ECOMMERCE", "OTHER"],
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import ResponseErrorResponse

from ..operations.extend_files import GetExtendFile


@same_doc_as(GetExtendFile)
def get_extend_file(
    file_path: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Retrieve extend hosted static files (GetExtendFile)

    Serves static files provided for extend features, for example: serving App UI files. Files are served with immutable caching headers.

    Properties:
        url: /csm/v1/admin/namespaces/{namespace}/files/{filePath}

        method: GET

        tags: ["Extend Files"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        file_path: (filePath) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - (OK)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        404: Not Found - ResponseErrorResponse (Not Found)

        502: Bad Gateway - ResponseErrorResponse (Bad Gateway)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = GetExtendFile.create(
        file_path=file_path,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetExtendFile)
async def get_extend_file_async(
    file_path: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Retrieve extend hosted static files (GetExtendFile)

    Serves static files provided for extend features, for example: serving App UI files. Files are served with immutable caching headers.

    Properties:
        url: /csm/v1/admin/namespaces/{namespace}/files/{filePath}

        method: GET

        tags: ["Extend Files"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        file_path: (filePath) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - (OK)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        404: Not Found - ResponseErrorResponse (Not Found)

        502: Bad Gateway - ResponseErrorResponse (Bad Gateway)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = GetExtendFile.create(
        file_path=file_path,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Platform Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.action_request import ActionRequest


class TradeChainedActionCommitRequest(Model):
    """Represents the request to commit a chained trade action. (TradeChainedActionCommitRequest)

    Properties:
        actions: (actions) OPTIONAL List[ActionRequest]

        metadata: (metadata) OPTIONAL Dict[str, Any]

        need_pre_check: (needPreCheck) OPTIONAL bool

        transaction_id: (transactionId) OPTIONAL str

        type_: (type) OPTIONAL str
    """

    # region fields

    actions: List[ActionRequest]  # OPTIONAL
    metadata: Dict[str, Any]  # OPTIONAL
    need_pre_check: bool  # OPTIONAL
    transaction_id: str  # OPTIONAL
    type_: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_actions(
        self, value: List[ActionRequest]
    ) -> TradeChainedActionCommitRequest:
        self.actions = value
        return self

    def with_metadata(self, value: Dict[str, Any]) -> TradeChainedActionCommitRequest:
        self.metadata = value
        return self

    def with_need_pre_check(self, value: bool) -> TradeChainedActionCommitRequest:
        self.need_pre_check = value
        return self

    def with_transaction_id(self, value: str) -> TradeChainedActionCommitRequest:
        self.transaction_id = value
        return self

    def with_type(self, value: str) -> TradeChainedActionCommitRequest:
        self.type_ = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "actions"):
            result["actions"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.actions
            ]
        elif include_empty:
            result["actions"] = []
        if hasattr(self, "metadata"):
            result["metadata"] = {str(k0): v0 for k0, v0 in self.metadata.items()}
        elif include_empty:
            result["metadata"] = {}
        if hasattr(self, "need_pre_check"):
            result["needPreCheck"] = bool(self.need_pre_check)
        elif include_empty:
            result["needPreCheck"] = False
        if hasattr(self, "transaction_id"):
            result["transactionId"] = str(self.transaction_id)
        elif include_empty:
            result["transactionId"] = ""
        if hasattr(self, "type_"):
            result["type"] = str(self.type_)
        elif include_empty:
            result["type"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        actions: Optional[List[ActionRequest]] = None,
        metadata: Optional[Dict[str, Any]] = None,
        need_pre_check: Optional[bool] = None,
        transaction_id: Optional[str] = None,
        type_: Optional[str] = None,
        **kwargs,
    ) -> TradeChainedActionCommitRequest:
        instance = cls()
        if actions is not None:
            instance.actions = actions
        if metadata is not None:
            instance.metadata = metadata
        if need_pre_check is not None:
            instance.need_pre_check = need_pre_check
        if transaction_id is not None:
            instance.transaction_id = transaction_id
        if type_ is not None:
            instance.type_ = type_
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> TradeChainedActionCommitRequest:
        instance = cls()
        if not dict_:
            return instance
        if "actions" in dict_ and dict_["actions"] is not None:
            instance.actions = [
                ActionRequest.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["actions"]
            ]
        elif include_empty:
            instance.actions = []
        if "metadata" in dict_ and dict_["metadata"] is not None:
            instance.metadata = {str(k0): v0 for k0, v0 in dict_["metadata"].items()}
        elif include_empty:
            instance.metadata = {}
        if "needPreCheck" in dict_ and dict_["needPreCheck"] is not None:
            instance.need_pre_check = bool(dict_["needPreCheck"])
        elif include_empty:
            instance.need_pre_check = False
        if "transactionId" in dict_ and dict_["transactionId"] is not None:
            instance.transaction_id = str(dict_["transactionId"])
        elif include_empty:
            instance.transaction_id = ""
        if "type" in dict_ and dict_["type"] is not None:
            instance.type_ = str(dict_["type"])
        elif include_empty:
            instance.type_ = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, TradeChainedActionCommitRequest]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[TradeChainedActionCommitRequest]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        TradeChainedActionCommitRequest,
        List[TradeChainedActionCommitRequest],
        Dict[Any, TradeChainedActionCommitRequest],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "actions": "actions",
            "metadata": "metadata",
            "needPreCheck": "need_pre_check",
            "transactionId": "transaction_id",
            "type": "type_",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "actions": False,
            "metadata": False,
            "needPreCheck": False,
            "transactionId": False,
            "type": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Login Queue Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from .utils import randomize

from ..api.loginqueue.models import ApimodelsConfigurationRequest
from ..api.loginqueue.models import ApimodelsConfigurationResponse
from ..api.loginqueue.models import ApimodelsLink
from ..api.loginqueue.models import ApimodelsQueueStatusResponse
from ..api.loginqueue.models import ApimodelsRefreshTicketResponse
from ..api.loginqueue.models import ResponseError


def create_apimodels_configuration_request_example() -> ApimodelsConfigurationRequest:
    instance = ApimodelsConfigurationRequest()
    instance.enabled = randomize("bool")
    instance.max_concurrency = randomize("int", min_val=1, max_val=1000)
    instance.max_login_rate = randomize("int", min_val=1, max_val=1000)
    instance.min_activation_period_in_second = randomize("int", min_val=1, max_val=1000)
    instance.player_polling_time_in_second = randomize("int", min_val=1, max_val=1000)
    instance.player_reconnect_grace_period_in_second = randomize(
        "int", min_val=1, max_val=1000
    )
    instance.queue_reconnect_grace_period_in_second = randomize(
        "int", min_val=1, max_val=1000
    )
    instance.safety_margin_percentage = randomize("int", min_val=1, max_val=1000)
    return instance


def create_apimodels_configuration_response_example() -> ApimodelsConfigurationResponse:
    instance = ApimodelsConfigurationResponse()
    instance.enabled = randomize("bool")
    instance.max_concurrency = randomize("int", min_val=1, max_val=1000)
    instance.max_login_rate = randomize("int", min_val=1, max_val=1000)
    instance.min_activation_period_in_second = randomize("int", min_val=1, max_val=1000)
    instance.namespace = randomize("slug")
    instance.player_polling_time_in_second = randomize("int", min_val=1, max_val=1000)
    instance.player_reconnect_grace_period_in_second = randomize(
        "int", min_val=1, max_val=1000
    )
    instance.queue_reconnect_grace_period_in_second = randomize(
        "int", min_val=1, max_val=1000
    )
    instance.safety_margin_percentage = randomize("int", min_val=1, max_val=1000)
    return instance


def create_apimodels_link_example() -> ApimodelsLink:
    instance = ApimodelsLink()
    instance.action = randomize()
    instance.href = randomize()
    return instance


def create_apimodels_queue_status_response_example() -> ApimodelsQueueStatusResponse:
    instance = ApimodelsQueueStatusResponse()
    instance.is_active = randomize("bool")
    instance.logged_user_count = randomize("int", min_val=1, max_val=1000)
    instance.login_rps = randomize("int", min_val=1, max_val=1000)
    instance.namespace = randomize("slug")
    instance.queue_count = randomize("int", min_val=1, max_val=1000)
    return instance


def create_apimodels_refresh_ticket_response_example() -> (
    ApimodelsRefreshTicketResponse
):
    instance = ApimodelsRefreshTicketResponse()
    instance.cancel = create_apimodels_link_example()
    instance.estimated_waiting_time_in_seconds = randomize(
        "int", min_val=1, max_val=1000
    )
    instance.namespace = randomize("slug")
    instance.player_polling_time_in_seconds = randomize("int", min_val=1, max_val=1000)
    instance.position = randomize("int", min_val=1, max_val=1000)
    instance.reconnect_expired_at = randomize("int", min_val=1, max_val=1000)
    return instance


def create_response_error_example() -> ResponseError:
    instance = ResponseError()
    instance.error_code = randomize("int", min_val=1, max_val=1000)
    instance.error_message = randomize()
    instance.message = randomize()
    instance.name = randomize()
    instance.attributes = {randomize(): randomize()}
    return instance



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import ModelsBinaryRecordRequest
from ..models import ModelsListPlayerBinaryRecordsResponse
from ..models import ModelsPlayerBinaryRecordCreate
from ..models import ModelsPlayerBinaryRecordMetadataRequest
from ..models import ModelsPlayerBinaryRecordResponse
from ..models import ModelsResponseError
from ..models import ModelsUploadBinaryRecordRequest
from ..models import ModelsUploadBinaryRecordResponse

from ..operations.admin_player_binary_record import AdminDeletePlayerBinaryRecordV1
from ..operations.admin_player_binary_record import AdminGetPlayerBinaryRecordV1
from ..operations.admin_player_binary_record import AdminListPlayerBinaryRecordsV1
from ..operations.admin_player_binary_record import AdminPostPlayerBinaryPresignedURLV1
from ..operations.admin_player_binary_record import AdminPostPlayerBinaryRecordV1
from ..operations.admin_player_binary_record import AdminPutPlayerBinaryRecorMetadataV1
from ..operations.admin_player_binary_record import AdminPutPlayerBinaryRecordV1
from ..models import ModelsPlayerBinaryRecordCreateSetByEnum
from ..models import ModelsPlayerBinaryRecordMetadataRequestSetByEnum
from ..models import ModelsPlayerBinaryRecordResponseSetByEnum


@same_doc_as(AdminDeletePlayerBinaryRecordV1)
def admin_delete_player_binary_record_v1(
    key: str,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete player binary record (adminDeletePlayerBinaryRecordV1)

    Delete a player binary record.

    Properties:
        url: /cloudsave/v1/admin/namespaces/{namespace}/users/{userId}/binaries/{key}

        method: DELETE

        tags: ["AdminPlayerBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        key: (key) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        204: No Content - (Record deleted)

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        404: Not Found - ModelsResponseError (18338: record not found)

        500: Internal Server Error - ModelsResponseError (18336: unable to delete record | 18338: record not found)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminDeletePlayerBinaryRecordV1.create(
        key=key,
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminDeletePlayerBinaryRecordV1)
async def admin_delete_player_binary_record_v1_async(
    key: str,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete player binary record (adminDeletePlayerBinaryRecordV1)

    Delete a player binary record.

    Properties:
        url: /cloudsave/v1/admin/namespaces/{namespace}/users/{userId}/binaries/{key}

        method: DELETE

        tags: ["AdminPlayerBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        key: (key) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        204: No Content - (Record deleted)

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        404: Not Found - ModelsResponseError (18338: record not found)

        500: Internal Server Error - ModelsResponseError (18336: unable to delete record | 18338: record not found)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminDeletePlayerBinaryRecordV1.create(
        key=key,
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminGetPlayerBinaryRecordV1)
def admin_get_player_binary_record_v1(
    key: str,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get player binary record (adminGetPlayerBinaryRecordV1)

    Get a player binary record by its key.

    Properties:
        url: /cloudsave/v1/admin/namespaces/{namespace}/users/{userId}/binaries/{key}

        method: GET

        tags: ["AdminPlayerBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        key: (key) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - ModelsPlayerBinaryRecordResponse (Record in user-level retrieved)

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        404: Not Found - ModelsResponseError (18325: record not found)

        500: Internal Server Error - ModelsResponseError (18323: unable to get record)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminGetPlayerBinaryRecordV1.create(
        key=key,
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminGetPlayerBinaryRecordV1)
async def admin_get_player_binary_record_v1_async(
    key: str,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get player binary record (adminGetPlayerBinaryRecordV1)

    Get a player binary record by its key.

    Properties:
        url: /cloudsave/v1/admin/namespaces/{namespace}/users/{userId}/binaries/{key}

        method: GET

        tags: ["AdminPlayerBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        key: (key) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - ModelsPlayerBinaryRecordResponse (Record in user-level retrieved)

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        404: Not Found - ModelsResponseError (18325: record not found)

        500: Internal Server Error - ModelsResponseError (18323: unable to get record)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminGetPlayerBinaryRecordV1.create(
        key=key,
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminListPlayerBinaryRecordsV1)
def admin_list_player_binary_records_v1(
    user_id: str,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    query: Optional[str] = None,
    tags: Optional[List[str]] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Query player binary records (adminListPlayerBinaryRecordsV1)

    Retrieve list of player binary records by namespace.

    Properties:
        url: /cloudsave/v1/admin/namespaces/{namespace}/users/{userId}/binaries

        method: GET

        tags: ["AdminPlayerBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        query: (query) OPTIONAL str in query

        tags: (tags) OPTIONAL List[str] in query

    Responses:
        200: OK - ModelsListPlayerBinaryRecordsResponse (Retrieve list of player records by namespace)

        400: Bad Request - ModelsResponseError (18326: invalid request body)

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        500: Internal Server Error - ModelsResponseError (18323: unable to get record)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminListPlayerBinaryRecordsV1.create(
        user_id=user_id,
        limit=limit,
        offset=offset,
        query=query,
        tags=tags,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminListPlayerBinaryRecordsV1)
async def admin_list_player_binary_records_v1_async(
    user_id: str,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    query: Optional[str] = None,
    tags: Optional[List[str]] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Query player binary records (adminListPlayerBinaryRecordsV1)

    Retrieve list of player binary records by namespace.

    Properties:
        url: /cloudsave/v1/admin/namespaces/{namespace}/users/{userId}/binaries

        method: GET

        tags: ["AdminPlayerBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        query: (query) OPTIONAL str in query

        tags: (tags) OPTIONAL List[str] in query

    Responses:
        200: OK - ModelsListPlayerBinaryRecordsResponse (Retrieve list of player records by namespace)

        400: Bad Request - ModelsResponseError (18326: invalid request body)

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        500: Internal Server Error - ModelsResponseError (18323: unable to get record)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminListPlayerBinaryRecordsV1.create(
        user_id=user_id,
        limit=limit,
        offset=offset,
        query=query,
        tags=tags,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminPostPlayerBinaryPresignedURLV1)
def admin_post_player_binary_presigned_urlv1(
    body: ModelsUploadBinaryRecordRequest,
    key: str,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Request presigned URL for upload player binary records (adminPostPlayerBinaryPresignedURLV1)

    Request presigned URL to upload the binary record to s3.

    Other detail info:

    Supported file types: jpeg, jpg, png, bmp, gif, mp3, webp, and bin.

    Properties:
        url: /cloudsave/v1/admin/namespaces/{namespace}/users/{userId}/binaries/{key}/presigned

        method: POST

        tags: ["AdminPlayerBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsUploadBinaryRecordRequest in body

        key: (key) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        201: Created - ModelsUploadBinaryRecordResponse (Successful Operation)

        400: Bad Request - ModelsResponseError (18311: invalid request body)

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        404: Not Found - ModelsResponseError (18313: record not found)

        500: Internal Server Error - ModelsResponseError (18312: unable to get record | 18314: unable to get presigned URL)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminPostPlayerBinaryPresignedURLV1.create(
        body=body,
        key=key,
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminPostPlayerBinaryPresignedURLV1)
async def admin_post_player_binary_presigned_urlv1_async(
    body: ModelsUploadBinaryRecordRequest,
    key: str,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Request presigned URL for upload player binary records (adminPostPlayerBinaryPresignedURLV1)

    Request presigned URL to upload the binary record to s3.

    Other detail info:

    Supported file types: jpeg, jpg, png, bmp, gif, mp3, webp, and bin.

    Properties:
        url: /cloudsave/v1/admin/namespaces/{namespace}/users/{userId}/binaries/{key}/presigned

        method: POST

        tags: ["AdminPlayerBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsUploadBinaryRecordRequest in body

        key: (key) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        201: Created - ModelsUploadBinaryRecordResponse (Successful Operation)

        400: Bad Request - ModelsResponseError (18311: invalid request body)

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        404: Not Found - ModelsResponseError (18313: record not found)

        500: Internal Server Error - ModelsResponseError (18312: unable to get record | 18314: unable to get presigned URL)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminPostPlayerBinaryPresignedURLV1.create(
        body=body,
        key=key,
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminPostPlayerBinaryRecordV1)
def admin_post_player_binary_record_v1(
    body: ModelsPlayerBinaryRecordCreate,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create player binary record (adminPostPlayerBinaryRecordV1)

    Create a player binary record.

    Other detail info:

    `key` should follow these rules:
    1. support uppercase and lowercase letters, numbers, and separators **"-"**, **"_"**, **"."** are allowed
    2. begin and end with letters or numbers
    3. spaces are not allowed
    4. separators must not appears twice in a row


    Supported file types: jpeg, jpg, png, bmp, gif, mp3, webp, and bin.

    Properties:
        url: /cloudsave/v1/admin/namespaces/{namespace}/users/{userId}/binaries

        method: POST

        tags: ["AdminPlayerBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsPlayerBinaryRecordCreate in body

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        201: Created - ModelsUploadBinaryRecordResponse (Record in user-level created)

        400: Bad Request - ModelsResponseError (18327: invalid request body)

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        409: Conflict - ModelsResponseError (18330: key already exists)

        500: Internal Server Error - ModelsResponseError (18323: unable to get record | 18328: unable to save record | 18331: unable to get presigned URL)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminPostPlayerBinaryRecordV1.create(
        body=body,
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminPostPlayerBinaryRecordV1)
async def admin_post_player_binary_record_v1_async(
    body: ModelsPlayerBinaryRecordCreate,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create player binary record (adminPostPlayerBinaryRecordV1)

    Create a player binary record.

    Other detail info:

    `key` should follow these rules:
    1. support uppercase and lowercase letters, numbers, and separators **"-"**, **"_"**, **"."** are allowed
    2. begin and end with letters or numbers
    3. spaces are not allowed
    4. separators must not appears twice in a row


    Supported file types: jpeg, jpg, png, bmp, gif, mp3, webp, and bin.

    Properties:
        url: /cloudsave/v1/admin/namespaces/{namespace}/users/{userId}/binaries

        method: POST

        tags: ["AdminPlayerBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsPlayerBinaryRecordCreate in body

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        201: Created - ModelsUploadBinaryRecordResponse (Record in user-level created)

        400: Bad Request - ModelsResponseError (18327: invalid request body)

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        409: Conflict - ModelsResponseError (18330: key already exists)

        500: Internal Server Error - ModelsResponseError (18323: unable to get record | 18328: unable to save record | 18331: unable to get presigned URL)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminPostPlayerBinaryRecordV1.create(
        body=body,
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminPutPlayerBinaryRecorMetadataV1)
def admin_put_player_binary_recor_metadata_v1(
    body: ModelsPlayerBinaryRecordMetadataRequest,
    key: str,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update player binary record metadata (adminPutPlayerBinaryRecorMetadataV1)

    Update a player binary record metadata by its key

    Properties:
        url: /cloudsave/v1/admin/namespaces/{namespace}/users/{userId}/binaries/{key}/metadata

        method: PUT

        tags: ["AdminPlayerBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsPlayerBinaryRecordMetadataRequest in body

        key: (key) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - ModelsPlayerBinaryRecordResponse (Record saved)

        400: Bad Request - ModelsResponseError (18332: invalid request body)

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        404: Not Found - ModelsResponseError (18333: record not found)

        500: Internal Server Error - ModelsResponseError (18334: unable to update record)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminPutPlayerBinaryRecorMetadataV1.create(
        body=body,
        key=key,
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminPutPlayerBinaryRecorMetadataV1)
async def admin_put_player_binary_recor_metadata_v1_async(
    body: ModelsPlayerBinaryRecordMetadataRequest,
    key: str,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update player binary record metadata (adminPutPlayerBinaryRecorMetadataV1)

    Update a player binary record metadata by its key

    Properties:
        url: /cloudsave/v1/admin/namespaces/{namespace}/users/{userId}/binaries/{key}/metadata

        method: PUT

        tags: ["AdminPlayerBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsPlayerBinaryRecordMetadataRequest in body

        key: (key) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - ModelsPlayerBinaryRecordResponse (Record saved)

        400: Bad Request - ModelsResponseError (18332: invalid request body)

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        404: Not Found - ModelsResponseError (18333: record not found)

        500: Internal Server Error - ModelsResponseError (18334: unable to update record)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminPutPlayerBinaryRecorMetadataV1.create(
        body=body,
        key=key,
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminPutPlayerBinaryRecordV1)
def admin_put_player_binary_record_v1(
    body: ModelsBinaryRecordRequest,
    key: str,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update player binary record file (adminPutPlayerBinaryRecordV1)

    Update a player binary record file by its key

    Properties:
        url: /cloudsave/v1/admin/namespaces/{namespace}/users/{userId}/binaries/{key}

        method: PUT

        tags: ["AdminPlayerBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsBinaryRecordRequest in body

        key: (key) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - ModelsPlayerBinaryRecordResponse (Record saved)

        400: Bad Request - ModelsResponseError (18332: invalid request body)

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        404: Not Found - ModelsResponseError (18333: record not found)

        500: Internal Server Error - ModelsResponseError (18334: unable to update record)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminPutPlayerBinaryRecordV1.create(
        body=body,
        key=key,
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminPutPlayerBinaryRecordV1)
async def admin_put_player_binary_record_v1_async(
    body: ModelsBinaryRecordRequest,
    key: str,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update player binary record file (adminPutPlayerBinaryRecordV1)

    Update a player binary record file by its key

    Properties:
        url: /cloudsave/v1/admin/namespaces/{namespace}/users/{userId}/binaries/{key}

        method: PUT

        tags: ["AdminPlayerBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsBinaryRecordRequest in body

        key: (key) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - ModelsPlayerBinaryRecordResponse (Record saved)

        400: Bad Request - ModelsResponseError (18332: invalid request body)

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        404: Not Found - ModelsResponseError (18333: record not found)

        500: Internal Server Error - ModelsResponseError (18334: unable to update record)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminPutPlayerBinaryRecordV1.create(
        body=body,
        key=key,
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import ModelsCreateTagRequest
from ..models import ModelsCreateTagResponse
from ..models import ModelsPaginatedGetTagResponse
from ..models import ResponseError

from ..operations.admin_tag import AdminCreateTag
from ..operations.admin_tag import AdminDeleteTag
from ..operations.admin_tag import AdminGetTag
from ..operations.admin_tag import AdminUpdateTag


@same_doc_as(AdminCreateTag)
def admin_create_tag(
    body: ModelsCreateTagRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create tags (AdminCreateTag)

    Creates a new tag

    Properties:
        url: /ugc/v1/admin/namespaces/{namespace}/tags

        method: POST

        tags: ["Admin Tag"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsCreateTagRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - ModelsCreateTagResponse (Tags created)

        400: Bad Request - ResponseError (771401: Malformed request/Invalid request body)

        401: Unauthorized - ResponseError (20001: unauthorized access)

        409: Conflict - ResponseError (771403: Conflicted resource indentifier)

        500: Internal Server Error - ResponseError (771402: Unable to save ugc tag)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminCreateTag.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminCreateTag)
async def admin_create_tag_async(
    body: ModelsCreateTagRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create tags (AdminCreateTag)

    Creates a new tag

    Properties:
        url: /ugc/v1/admin/namespaces/{namespace}/tags

        method: POST

        tags: ["Admin Tag"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsCreateTagRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - ModelsCreateTagResponse (Tags created)

        400: Bad Request - ResponseError (771401: Malformed request/Invalid request body)

        401: Unauthorized - ResponseError (20001: unauthorized access)

        409: Conflict - ResponseError (771403: Conflicted resource indentifier)

        500: Internal Server Error - ResponseError (771402: Unable to save ugc tag)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminCreateTag.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminDeleteTag)
def admin_delete_tag(
    tag_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete tags (AdminDeleteTag)

    Delete existing tag

    Properties:
        url: /ugc/v1/admin/namespaces/{namespace}/tags/{tagId}

        method: DELETE

        tags: ["Admin Tag"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        tag_id: (tagId) REQUIRED str in path

    Responses:
        204: No Content - (Tags deleted)

        401: Unauthorized - ResponseError (20001: unauthorized access)

        404: Not Found - ResponseError (772003: Tag not found)

        500: Internal Server Error - ResponseError (772002: Unable delete tag)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminDeleteTag.create(
        tag_id=tag_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminDeleteTag)
async def admin_delete_tag_async(
    tag_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete tags (AdminDeleteTag)

    Delete existing tag

    Properties:
        url: /ugc/v1/admin/namespaces/{namespace}/tags/{tagId}

        method: DELETE

        tags: ["Admin Tag"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        tag_id: (tagId) REQUIRED str in path

    Responses:
        204: No Content - (Tags deleted)

        401: Unauthorized - ResponseError (20001: unauthorized access)

        404: Not Found - ResponseError (772003: Tag not found)

        500: Internal Server Error - ResponseError (772002: Unable delete tag)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminDeleteTag.create(
        tag_id=tag_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminGetTag)
def admin_get_tag(
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get tags (AdminGetTag)

    Get available tags paginated

    Properties:
        url: /ugc/v1/admin/namespaces/{namespace}/tags

        method: GET

        tags: ["Admin Tag"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

    Responses:
        200: OK - ModelsPaginatedGetTagResponse (Get Tags)

        400: Bad Request - ResponseError (771501: invalid paging parameter)

        401: Unauthorized - ResponseError (20001: unauthorized access)

        500: Internal Server Error - ResponseError (771402: Unable to save ugc tag)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminGetTag.create(
        limit=limit,
        offset=offset,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminGetTag)
async def admin_get_tag_async(
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get tags (AdminGetTag)

    Get available tags paginated

    Properties:
        url: /ugc/v1/admin/namespaces/{namespace}/tags

        method: GET

        tags: ["Admin Tag"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

    Responses:
        200: OK - ModelsPaginatedGetTagResponse (Get Tags)

        400: Bad Request - ResponseError (771501: invalid paging parameter)

        401: Unauthorized - ResponseError (20001: unauthorized access)

        500: Internal Server Error - ResponseError (771402: Unable to save ugc tag)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminGetTag.create(
        limit=limit,
        offset=offset,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminUpdateTag)
def admin_update_tag(
    body: ModelsCreateTagRequest,
    tag_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update tag (AdminUpdateTag)

    Update existing tag

    Properties:
        url: /ugc/v1/admin/namespaces/{namespace}/tags/{tagId}

        method: PUT

        tags: ["Admin Tag"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsCreateTagRequest in body

        namespace: (namespace) REQUIRED str in path

        tag_id: (tagId) REQUIRED str in path

    Responses:
        200: OK - ModelsCreateTagResponse (Tags updated)

        400: Bad Request - ResponseError (772801: Malformed request/Invalid request body)

        401: Unauthorized - ResponseError (20001: unauthorized access)

        404: Not Found - ResponseError (772803: Tag not found)

        409: Conflict - ResponseError (772804: Proposed Tag already exist)

        500: Internal Server Error - ResponseError (772802: Unable update tags)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminUpdateTag.create(
        body=body,
        tag_id=tag_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminUpdateTag)
async def admin_update_tag_async(
    body: ModelsCreateTagRequest,
    tag_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update tag (AdminUpdateTag)

    Update existing tag

    Properties:
        url: /ugc/v1/admin/namespaces/{namespace}/tags/{tagId}

        method: PUT

        tags: ["Admin Tag"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsCreateTagRequest in body

        namespace: (namespace) REQUIRED str in path

        tag_id: (tagId) REQUIRED str in path

    Responses:
        200: OK - ModelsCreateTagResponse (Tags updated)

        400: Bad Request - ResponseError (772801: Malformed request/Invalid request body)

        401: Unauthorized - ResponseError (20001: unauthorized access)

        404: Not Found - ResponseError (772803: Tag not found)

        409: Conflict - ResponseError (772804: Proposed Tag already exist)

        500: Internal Server Error - ResponseError (772802: Unable update tags)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminUpdateTag.create(
        body=body,
        tag_id=tag_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Iam Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.rest_error_response_with_conflicted_user_platform_accounts import (
    RestErrorResponseWithConflictedUserPlatformAccounts,
)


class ModelLinkRequestResponse(Model):
    """Model link request response (model.LinkRequestResponse)

    Properties:
        client_id: (clientId) REQUIRED str

        namespace: (namespace) REQUIRED str

        payload: (payload) REQUIRED Dict[str, Any]

        redirect_uri: (redirectUri) REQUIRED str

        request_id: (requestId) REQUIRED str

        status: (status) REQUIRED str

        conflict_publisher_user_id: (conflictPublisherUserId) OPTIONAL str

        conflict_user_linked_games: (conflictUserLinkedGames) OPTIONAL List[str]

        current_user_linked_games: (currentUserLinkedGames) OPTIONAL List[str]

        error: (error) OPTIONAL RestErrorResponseWithConflictedUserPlatformAccounts

        expiration: (expiration) OPTIONAL int

        platform_display_name: (platformDisplayName) OPTIONAL str

        platform_id: (platformId) OPTIONAL str

        platform_user_id: (platformUserId) OPTIONAL str

        user_id: (userId) OPTIONAL str
    """

    # region fields

    client_id: str  # REQUIRED
    namespace: str  # REQUIRED
    payload: Dict[str, Any]  # REQUIRED
    redirect_uri: str  # REQUIRED
    request_id: str  # REQUIRED
    status: str  # REQUIRED
    conflict_publisher_user_id: str  # OPTIONAL
    conflict_user_linked_games: List[str]  # OPTIONAL
    current_user_linked_games: List[str]  # OPTIONAL
    error: RestErrorResponseWithConflictedUserPlatformAccounts  # OPTIONAL
    expiration: int  # OPTIONAL
    platform_display_name: str  # OPTIONAL
    platform_id: str  # OPTIONAL
    platform_user_id: str  # OPTIONAL
    user_id: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_client_id(self, value: str) -> ModelLinkRequestResponse:
        self.client_id = value
        return self

    def with_namespace(self, value: str) -> ModelLinkRequestResponse:
        self.namespace = value
        return self

    def with_payload(self, value: Dict[str, Any]) -> ModelLinkRequestResponse:
        self.payload = value
        return self

    def with_redirect_uri(self, value: str) -> ModelLinkRequestResponse:
        self.redirect_uri = value
        return self

    def with_request_id(self, value: str) -> ModelLinkRequestResponse:
        self.request_id = value
        return self

    def with_status(self, value: str) -> ModelLinkRequestResponse:
        self.status = value
        return self

    def with_conflict_publisher_user_id(self, value: str) -> ModelLinkRequestResponse:
        self.conflict_publisher_user_id = value
        return self

    def with_conflict_user_linked_games(
        self, value: List[str]
    ) -> ModelLinkRequestResponse:
        self.conflict_user_linked_games = value
        return self

    def with_current_user_linked_games(
        self, value: List[str]
    ) -> ModelLinkRequestResponse:
        self.current_user_linked_games = value
        return self

    def with_error(
        self, value: RestErrorResponseWithConflictedUserPlatformAccounts
    ) -> ModelLinkRequestResponse:
        self.error = value
        return self

    def with_expiration(self, value: int) -> ModelLinkRequestResponse:
        self.expiration = value
        return self

    def with_platform_display_name(self, value: str) -> ModelLinkRequestResponse:
        self.platform_display_name = value
        return self

    def with_platform_id(self, value: str) -> ModelLinkRequestResponse:
        self.platform_id = value
        return self

    def with_platform_user_id(self, value: str) -> ModelLinkRequestResponse:
        self.platform_user_id = value
        return self

    def with_user_id(self, value: str) -> ModelLinkRequestResponse:
        self.user_id = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "client_id"):
            result["clientId"] = str(self.client_id)
        elif include_empty:
            result["clientId"] = ""
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "payload"):
            result["payload"] = {str(k0): v0 for k0, v0 in self.payload.items()}
        elif include_empty:
            result["payload"] = {}
        if hasattr(self, "redirect_uri"):
            result["redirectUri"] = str(self.redirect_uri)
        elif include_empty:
            result["redirectUri"] = ""
        if hasattr(self, "request_id"):
            result["requestId"] = str(self.request_id)
        elif include_empty:
            result["requestId"] = ""
        if hasattr(self, "status"):
            result["status"] = str(self.status)
        elif include_empty:
            result["status"] = ""
        if hasattr(self, "conflict_publisher_user_id"):
            result["conflictPublisherUserId"] = str(self.conflict_publisher_user_id)
        elif include_empty:
            result["conflictPublisherUserId"] = ""
        if hasattr(self, "conflict_user_linked_games"):
            result["conflictUserLinkedGames"] = [
                str(i0) for i0 in self.conflict_user_linked_games
            ]
        elif include_empty:
            result["conflictUserLinkedGames"] = []
        if hasattr(self, "current_user_linked_games"):
            result["currentUserLinkedGames"] = [
                str(i0) for i0 in self.current_user_linked_games
            ]
        elif include_empty:
            result["currentUserLinkedGames"] = []
        if hasattr(self, "error"):
            result["error"] = self.error.to_dict(include_empty=include_empty)
        elif include_empty:
            result["error"] = RestErrorResponseWithConflictedUserPlatformAccounts()
        if hasattr(self, "expiration"):
            result["expiration"] = int(self.expiration)
        elif include_empty:
            result["expiration"] = 0
        if hasattr(self, "platform_display_name"):
            result["platformDisplayName"] = str(self.platform_display_name)
        elif include_empty:
            result["platformDisplayName"] = ""
        if hasattr(self, "platform_id"):
            result["platformId"] = str(self.platform_id)
        elif include_empty:
            result["platformId"] = ""
        if hasattr(self, "platform_user_id"):
            result["platformUserId"] = str(self.platform_user_id)
        elif include_empty:
            result["platformUserId"] = ""
        if hasattr(self, "user_id"):
            result["userId"] = str(self.user_id)
        elif include_empty:
            result["userId"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        client_id: str,
        namespace: str,
        payload: Dict[str, Any],
        redirect_uri: str,
        request_id: str,
        status: str,
        conflict_publisher_user_id: Optional[str] = None,
        conflict_user_linked_games: Optional[List[str]] = None,
        current_user_linked_games: Optional[List[str]] = None,
        error: Optional[RestErrorResponseWithConflictedUserPlatformAccounts] = None,
        expiration: Optional[int] = None,
        platform_display_name: Optional[str] = None,
        platform_id: Optional[str] = None,
        platform_user_id: Optional[str] = None,
        user_id: Optional[str] = None,
        **kwargs,
    ) -> ModelLinkRequestResponse:
        instance = cls()
        instance.client_id = client_id
        instance.namespace = namespace
        instance.payload = payload
        instance.redirect_uri = redirect_uri
        instance.request_id = request_id
        instance.status = status
        if conflict_publisher_user_id is not None:
            instance.conflict_publisher_user_id = conflict_publisher_user_id
        if conflict_user_linked_games is not None:
            instance.conflict_user_linked_games = conflict_user_linked_games
        if current_user_linked_games is not None:
            instance.current_user_linked_games = current_user_linked_games
        if error is not None:
            instance.error = error
        if expiration is not None:
            instance.expiration = expiration
        if platform_display_name is not None:
            instance.platform_display_name = platform_display_name
        if platform_id is not None:
            instance.platform_id = platform_id
        if platform_user_id is not None:
            instance.platform_user_id = platform_user_id
        if user_id is not None:
            instance.user_id = user_id
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelLinkRequestResponse:
        instance = cls()
        if not dict_:
            return instance
        if "clientId" in dict_ and dict_["clientId"] is not None:
            instance.client_id = str(dict_["clientId"])
        elif include_empty:
            instance.client_id = ""
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "payload" in dict_ and dict_["payload"] is not None:
            instance.payload = {str(k0): v0 for k0, v0 in dict_["payload"].items()}
        elif include_empty:
            instance.payload = {}
        if "redirectUri" in dict_ and dict_["redirectUri"] is not None:
            instance.redirect_uri = str(dict_["redirectUri"])
        elif include_empty:
            instance.redirect_uri = ""
        if "requestId" in dict_ and dict_["requestId"] is not None:
            instance.request_id = str(dict_["requestId"])
        elif include_empty:
            instance.request_id = ""
        if "status" in dict_ and dict_["status"] is not None:
            instance.status = str(dict_["status"])
        elif include_empty:
            instance.status = ""
        if (
            "conflictPublisherUserId" in dict_
            and dict_["conflictPublisherUserId"] is not None
        ):
            instance.conflict_publisher_user_id = str(dict_["conflictPublisherUserId"])
        elif include_empty:
            instance.conflict_publisher_user_id = ""
        if (
            "conflictUserLinkedGames" in dict_
            and dict_["conflictUserLinkedGames"] is not None
        ):
            instance.conflict_user_linked_games = [
                str(i0) for i0 in dict_["conflictUserLinkedGames"]
            ]
        elif include_empty:
            instance.conflict_user_linked_games = []
        if (
            "currentUserLinkedGames" in dict_
            and dict_["currentUserLinkedGames"] is not None
        ):
            instance.current_user_linked_games = [
                str(i0) for i0 in dict_["currentUserLinkedGames"]
            ]
        elif include_empty:
            instance.current_user_linked_games = []
        if "error" in dict_ and dict_["error"] is not None:
            instance.error = (
                RestErrorResponseWithConflictedUserPlatformAccounts.create_from_dict(
                    dict_["error"], include_empty=include_empty
                )
            )
        elif include_empty:
            instance.error = RestErrorResponseWithConflictedUserPlatformAccounts()
        if "expiration" in dict_ and dict_["expiration"] is not None:
            instance.expiration = int(dict_["expiration"])
        elif include_empty:
            instance.expiration = 0
        if "platformDisplayName" in dict_ and dict_["platformDisplayName"] is not None:
            instance.platform_display_name = str(dict_["platformDisplayName"])
        elif include_empty:
            instance.platform_display_name = ""
        if "platformId" in dict_ and dict_["platformId"] is not None:
            instance.platform_id = str(dict_["platformId"])
        elif include_empty:
            instance.platform_id = ""
        if "platformUserId" in dict_ and dict_["platformUserId"] is not None:
            instance.platform_user_id = str(dict_["platformUserId"])
        elif include_empty:
            instance.platform_user_id = ""
        if "userId" in dict_ and dict_["userId"] is not None:
            instance.user_id = str(dict_["userId"])
        elif include_empty:
            instance.user_id = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelLinkRequestResponse]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelLinkRequestResponse]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ModelLinkRequestResponse,
        List[ModelLinkRequestResponse],
        Dict[Any, ModelLinkRequestResponse],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "clientId": "client_id",
            "namespace": "namespace",
            "payload": "payload",
            "redirectUri": "redirect_uri",
            "requestId": "request_id",
            "status": "status",
            "conflictPublisherUserId": "conflict_publisher_user_id",
            "conflictUserLinkedGames": "conflict_user_linked_games",
            "currentUserLinkedGames": "current_user_linked_games",
            "error": "error",
            "expiration": "expiration",
            "platformDisplayName": "platform_display_name",
            "platformId": "platform_id",
            "platformUserId": "platform_user_id",
            "userId": "user_id",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "clientId": True,
            "namespace": True,
            "payload": True,
            "redirectUri": True,
            "requestId": True,
            "status": True,
            "conflictPublisherUserId": False,
            "conflictUserLinkedGames": False,
            "currentUserLinkedGames": False,
            "error": False,
            "expiration": False,
            "platformDisplayName": False,
            "platformId": False,
            "platformUserId": False,
            "userId": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import ModelsPluginRequest
from ..models import ModelsPluginResponse
from ..models import ModelsResponseError

from ..operations.plugin_config import CreatePluginConfig
from ..operations.plugin_config import DeletePluginConfig
from ..operations.plugin_config import GetPluginConfig
from ..operations.plugin_config import UpdatePluginConfig
from ..models import ModelsPluginRequestExtendTypeEnum
from ..models import ModelsPluginResponseExtendTypeEnum


@same_doc_as(CreatePluginConfig)
def create_plugin_config(
    body: ModelsPluginRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create plugin configuration (createPluginConfig)

    ## Description

    This endpoints will create new grpc plugins configuration per namespace

    Properties:
        url: /cloudsave/v1/admin/namespaces/{namespace}/plugins

        method: POST

        tags: ["PluginConfig"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsPluginRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - ModelsPluginResponse (Created)

        400: Bad Request - ModelsResponseError (18401: invalid request body | 20002: validation error)

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        409: Conflict - ModelsResponseError (18402: plugins already configured)

        500: Internal Server Error - ModelsResponseError (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = CreatePluginConfig.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(CreatePluginConfig)
async def create_plugin_config_async(
    body: ModelsPluginRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create plugin configuration (createPluginConfig)

    ## Description

    This endpoints will create new grpc plugins configuration per namespace

    Properties:
        url: /cloudsave/v1/admin/namespaces/{namespace}/plugins

        method: POST

        tags: ["PluginConfig"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsPluginRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - ModelsPluginResponse (Created)

        400: Bad Request - ModelsResponseError (18401: invalid request body | 20002: validation error)

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        409: Conflict - ModelsResponseError (18402: plugins already configured)

        500: Internal Server Error - ModelsResponseError (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = CreatePluginConfig.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(DeletePluginConfig)
def delete_plugin_config(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete plugin configuration (deletePluginConfig)

    ## Description

    This endpoints will delete grpc plugins configuration

    Properties:
        url: /cloudsave/v1/admin/namespaces/{namespace}/plugins

        method: DELETE

        tags: ["PluginConfig"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (OK)

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        404: Not Found - ModelsResponseError (18406: plugins config not found)

        500: Internal Server Error - ModelsResponseError (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = DeletePluginConfig.create(
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(DeletePluginConfig)
async def delete_plugin_config_async(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete plugin configuration (deletePluginConfig)

    ## Description

    This endpoints will delete grpc plugins configuration

    Properties:
        url: /cloudsave/v1/admin/namespaces/{namespace}/plugins

        method: DELETE

        tags: ["PluginConfig"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (OK)

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        404: Not Found - ModelsResponseError (18406: plugins config not found)

        500: Internal Server Error - ModelsResponseError (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = DeletePluginConfig.create(
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetPluginConfig)
def get_plugin_config(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get plugin configuration (getPluginConfig)

    ## Description

    This endpoints will get grpc plugins configuration

    Properties:
        url: /cloudsave/v1/admin/namespaces/{namespace}/plugins

        method: GET

        tags: ["PluginConfig"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelsPluginResponse (OK)

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        404: Not Found - ModelsResponseError (18404: plugins not found)

        500: Internal Server Error - ModelsResponseError (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = GetPluginConfig.create(
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetPluginConfig)
async def get_plugin_config_async(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get plugin configuration (getPluginConfig)

    ## Description

    This endpoints will get grpc plugins configuration

    Properties:
        url: /cloudsave/v1/admin/namespaces/{namespace}/plugins

        method: GET

        tags: ["PluginConfig"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelsPluginResponse (OK)

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        404: Not Found - ModelsResponseError (18404: plugins not found)

        500: Internal Server Error - ModelsResponseError (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = GetPluginConfig.create(
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(UpdatePluginConfig)
def update_plugin_config(
    body: ModelsPluginRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update plugin configuration (updatePluginConfig)

    ## Description

    This endpoints will update grpc plugins configuration

    Properties:
        url: /cloudsave/v1/admin/namespaces/{namespace}/plugins

        method: PATCH

        tags: ["PluginConfig"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsPluginRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelsPluginResponse (OK)

        400: Bad Request - ModelsResponseError (18408: invalid request body | 20002: validation error)

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        404: Not Found - ModelsResponseError (18409: plugins config not found)

        500: Internal Server Error - ModelsResponseError (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = UpdatePluginConfig.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(UpdatePluginConfig)
async def update_plugin_config_async(
    body: ModelsPluginRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update plugin configuration (updatePluginConfig)

    ## Description

    This endpoints will update grpc plugins configuration

    Properties:
        url: /cloudsave/v1/admin/namespaces/{namespace}/plugins

        method: PATCH

        tags: ["PluginConfig"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsPluginRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelsPluginResponse (OK)

        400: Bad Request - ModelsResponseError (18408: invalid request body | 20002: validation error)

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        404: Not Found - ModelsResponseError (18409: plugins config not found)

        500: Internal Server Error - ModelsResponseError (20000: internal server error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = UpdatePluginConfig.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



from __future__ import annotations

from logging import Logger
from threading import RLock
from threading import Timer as ThreadingTimer
from typing import Any, Callable, Optional, Tuple, Union
from uuid import uuid4

from ._strenum import StrEnum


class TimerStatus(StrEnum):
    IDLE = "IDLE"
    RUNNING = "RUNNING"
    CANCELLED = "CANCELLED"
    FINISHED = "FINISHED"


class Timer:
    """Call a function repeatedly after a specified number of seconds:

        # name: t.name = "Foo"
        # daemon: t.daemon = True|False
        # repeats: <0: repeat 'inf' times
                   0|None: repeat 0 times
                   >0: repeat n times
        t = Timer(30.0, f, args=None, kwargs=None, name="Foo", daemon=False, repeats=10)
        t.start()
        t.cancel()     # stop the timer's action if it's still waiting

    # Uses 'threading.Timer' internally.
    """

    def __init__(
        self,
        interval: Union[int, float],
        function: Callable,
        /,
        *,
        args=None,
        kwargs=None,
        name: Optional[str] = None,
        daemon: bool = False,
        repeats: Optional[int] = None,
        autostart: bool = False,
        repeat_on_exception: bool = False,
        logger: Optional[Logger] = None,
    ) -> None:
        self._uid = uuid4().hex
        self._interval = interval
        self._function = function
        self._args = args if args is not None else []
        self._kwargs = kwargs if kwargs is not None else {}

        self._name: Optional[str] = name
        self._daemon: bool = daemon
        self._repeats: Optional[int] = repeats
        self._autostart: bool = autostart
        self._repeat_on_exception: bool = repeat_on_exception
        self._logger: Logger = logger

        self._lock: RLock = RLock()
        self._timer: Optional[ThreadingTimer] = None

        self._counter: int = 0
        self.__status: TimerStatus = TimerStatus.IDLE

        self._exception: Optional[Exception] = None
        self._result: Any = None

        if self._autostart:
            self.start()

    # noinspection PyProtectedMember, PyUnresolvedReferences
    def __repr__(self) -> str:
        class_name = "?"
        status = "idle"
        if self._repeats is None or self._repeats == 0:
            repeats = "1"
        elif self._repeats < 0:
            repeats = "inf"
        else:
            repeats = str(self._repeats + 1)
        if self._timer is not None:
            class_name = self._timer.__class__.__name__
            status = "initial"
            if self._timer._started.is_set():
                status = "started"
            self._timer.is_alive()
            if self._timer._is_stopped:
                status = "stopped"
            if self._timer._daemonic:
                status += " daemon"
            if self._timer._ident is not None:
                status += f" {self._timer._ident}"
        return "<{class_container_name}:{class_name}({uid}, {name}, {status})>[{counter}/{repeats}]".format(
            class_container_name=self.__class__.__name__,
            class_name=class_name,
            uid=self._uid,
            name=self.name,
            status=status,
            counter=self._counter,
            repeats=repeats,
        )

    @property
    def exception(self) -> Optional[Exception]:
        return self._exception

    # noinspection PyProtectedMember, PyUnresolvedReferences
    @property
    def name(self) -> str:
        if self._timer is not None:
            return self._timer._name
        else:
            default_name = "?"
            if self._name is not None:
                return "{}:{}".format(self._name, default_name)
            else:
                return default_name

    @property
    def result(self) -> Any:
        return self._result

    @property
    def status(self) -> TimerStatus:
        return self._status

    @property
    def _status(self) -> TimerStatus:
        with self._lock:
            return self.__status

    @_status.setter
    def _status(self, s: TimerStatus) -> None:
        with self._lock:
            self.__status = s

    def start(self, *args, **kwargs) -> Timer:
        self.reset()

        self._timer = self._create_timer(*args, **kwargs)
        self._timer.start()
        self._status = TimerStatus.RUNNING

        return self

    def cancel(self) -> Timer:
        if self._timer is not None:
            self._timer.cancel()
            self._status = TimerStatus.CANCELLED

        return self

    def reset(self) -> Timer:
        self.cancel()

        self._timer = None
        self._status = TimerStatus.IDLE

        self._result = None
        self._exception = None

        return self

    def clone(self, **kwargs) -> Timer:
        interval = kwargs.pop("interval", self._interval)

        kw = {
            "args": self._args,
            "kwargs": self._kwargs,
            "name": self._name,
            "daemon": self._daemon,
            "repeats": self._repeats,
            "autostart": self._autostart,
            "repeat_on_exception": self._repeat_on_exception,
        }

        for k, v in kwargs.items():
            if k not in kw:
                raise KeyError(k)
            kw[k] = v

        return Timer(interval, self._function, **kw)

    def _create_timer(self, *args, **kwargs) -> ThreadingTimer:
        timer = ThreadingTimer(
            self._interval, self._execute_function, args=args, kwargs=kwargs
        )

        if self._name is not None:
            timer.name = "{}:{}".format(self._name, timer.name)

        timer.daemon = self._daemon

        return timer

    def _execute_function(self, *args, **kwargs) -> None:
        self._counter += 1

        fargs = []
        fargs.extend(self._args)
        fargs.extend(args)

        fkwargs = {}
        fkwargs.update(self._kwargs)
        fkwargs.update(kwargs)

        has_exception = False
        try:
            result = self._function(*fargs, **fkwargs)
            self._result = result
        except Exception as exception:
            has_exception = True
            self._exception = exception
            if self._logger:
                self._logger.error(
                    f"{repr(self)} caught an exception: {self._exception}"
                )

        repeat, reason = self._should_repeat(has_exception=has_exception)
        if repeat:
            self.start()
        else:
            self._status = TimerStatus.FINISHED
            if self._logger and reason:
                self._logger.warning(
                    f"{repr(self)} status set to FINISHED due to {reason}"
                )

    def _should_repeat(self, has_exception: bool) -> Tuple[bool, str]:
        if self._status == TimerStatus.CANCELLED:
            return False, "being cancelled"
        if has_exception and not self._repeat_on_exception:
            return False, f"exception: {self._exception}"
        if self._repeats is None or self._repeats == 0:
            return False, "repeats aren't allowed"
        if self._repeats < 0:
            return True, ""
        return self._counter <= self._repeats, ""



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import PaymentAccount

from ..operations.payment_account import PublicDeletePaymentAccount
from ..operations.payment_account import PublicDeletePaymentAccountTypeEnum
from ..operations.payment_account import PublicGetPaymentAccounts
from ..models import PaymentAccountTypeEnum


@same_doc_as(PublicDeletePaymentAccount)
def public_delete_payment_account(
    id_: str,
    type_: Union[str, PublicDeletePaymentAccountTypeEnum],
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete payment account (publicDeletePaymentAccount)

    [Not supported yet in AGS Shared Cloud] Delete payment account.
    Other detail info:

      * Returns :

    Properties:
        url: /platform/public/namespaces/{namespace}/users/{userId}/payment/accounts/{type}/{id}

        method: DELETE

        tags: ["PaymentAccount"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        id_: (id) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        type_: (type) REQUIRED Union[str, TypeEnum] in path

        user_id: (userId) REQUIRED str in path

    Responses:
        204: No Content - (delete a payment account successfully)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = PublicDeletePaymentAccount.create(
        id_=id_,
        type_=type_,
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PublicDeletePaymentAccount)
async def public_delete_payment_account_async(
    id_: str,
    type_: Union[str, PublicDeletePaymentAccountTypeEnum],
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete payment account (publicDeletePaymentAccount)

    [Not supported yet in AGS Shared Cloud] Delete payment account.
    Other detail info:

      * Returns :

    Properties:
        url: /platform/public/namespaces/{namespace}/users/{userId}/payment/accounts/{type}/{id}

        method: DELETE

        tags: ["PaymentAccount"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        id_: (id) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        type_: (type) REQUIRED Union[str, TypeEnum] in path

        user_id: (userId) REQUIRED str in path

    Responses:
        204: No Content - (delete a payment account successfully)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = PublicDeletePaymentAccount.create(
        id_=id_,
        type_=type_,
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(PublicGetPaymentAccounts)
def public_get_payment_accounts(
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get payment accounts (publicGetPaymentAccounts)

    [Not supported yet in AGS Shared Cloud] Get payment accounts.
    Other detail info:

      * Returns : Payment account list

    Properties:
        url: /platform/public/namespaces/{namespace}/users/{userId}/payment/accounts

        method: GET

        tags: ["PaymentAccount"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - List[PaymentAccount] (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = PublicGetPaymentAccounts.create(
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PublicGetPaymentAccounts)
async def public_get_payment_accounts_async(
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get payment accounts (publicGetPaymentAccounts)

    [Not supported yet in AGS Shared Cloud] Get payment accounts.
    Other detail info:

      * Returns : Payment account list

    Properties:
        url: /platform/public/namespaces/{namespace}/users/{userId}/payment/accounts

        method: GET

        tags: ["PaymentAccount"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - List[PaymentAccount] (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = PublicGetPaymentAccounts.create(
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Iam Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.model_simple_user_ban import ModelSimpleUserBan
from ..models.platform_dlc_agg_response import PlatformDLCAggResponse
from ..models.platform_wallet_agg_response import PlatformWalletAggResponse


class ModelProgressionInfo(Model):
    """Model progression info (model.ProgressionInfo)

    Properties:
        created_at: (createdAt) REQUIRED int

        namespace: (namespace) REQUIRED str

        dlcs: (dlcs) OPTIONAL PlatformDLCAggResponse

        last_login_time: (lastLoginTime) OPTIONAL int

        user_ban: (userBan) OPTIONAL ModelSimpleUserBan

        wallets: (wallets) OPTIONAL List[PlatformWalletAggResponse]
    """

    # region fields

    created_at: int  # REQUIRED
    namespace: str  # REQUIRED
    dlcs: PlatformDLCAggResponse  # OPTIONAL
    last_login_time: int  # OPTIONAL
    user_ban: ModelSimpleUserBan  # OPTIONAL
    wallets: List[PlatformWalletAggResponse]  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_created_at(self, value: int) -> ModelProgressionInfo:
        self.created_at = value
        return self

    def with_namespace(self, value: str) -> ModelProgressionInfo:
        self.namespace = value
        return self

    def with_dlcs(self, value: PlatformDLCAggResponse) -> ModelProgressionInfo:
        self.dlcs = value
        return self

    def with_last_login_time(self, value: int) -> ModelProgressionInfo:
        self.last_login_time = value
        return self

    def with_user_ban(self, value: ModelSimpleUserBan) -> ModelProgressionInfo:
        self.user_ban = value
        return self

    def with_wallets(
        self, value: List[PlatformWalletAggResponse]
    ) -> ModelProgressionInfo:
        self.wallets = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "created_at"):
            result["createdAt"] = int(self.created_at)
        elif include_empty:
            result["createdAt"] = 0
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "dlcs"):
            result["dlcs"] = self.dlcs.to_dict(include_empty=include_empty)
        elif include_empty:
            result["dlcs"] = PlatformDLCAggResponse()
        if hasattr(self, "last_login_time"):
            result["lastLoginTime"] = int(self.last_login_time)
        elif include_empty:
            result["lastLoginTime"] = 0
        if hasattr(self, "user_ban"):
            result["userBan"] = self.user_ban.to_dict(include_empty=include_empty)
        elif include_empty:
            result["userBan"] = ModelSimpleUserBan()
        if hasattr(self, "wallets"):
            result["wallets"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.wallets
            ]
        elif include_empty:
            result["wallets"] = []
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        created_at: int,
        namespace: str,
        dlcs: Optional[PlatformDLCAggResponse] = None,
        last_login_time: Optional[int] = None,
        user_ban: Optional[ModelSimpleUserBan] = None,
        wallets: Optional[List[PlatformWalletAggResponse]] = None,
        **kwargs,
    ) -> ModelProgressionInfo:
        instance = cls()
        instance.created_at = created_at
        instance.namespace = namespace
        if dlcs is not None:
            instance.dlcs = dlcs
        if last_login_time is not None:
            instance.last_login_time = last_login_time
        if user_ban is not None:
            instance.user_ban = user_ban
        if wallets is not None:
            instance.wallets = wallets
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelProgressionInfo:
        instance = cls()
        if not dict_:
            return instance
        if "createdAt" in dict_ and dict_["createdAt"] is not None:
            instance.created_at = int(dict_["createdAt"])
        elif include_empty:
            instance.created_at = 0
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "dlcs" in dict_ and dict_["dlcs"] is not None:
            instance.dlcs = PlatformDLCAggResponse.create_from_dict(
                dict_["dlcs"], include_empty=include_empty
            )
        elif include_empty:
            instance.dlcs = PlatformDLCAggResponse()
        if "lastLoginTime" in dict_ and dict_["lastLoginTime"] is not None:
            instance.last_login_time = int(dict_["lastLoginTime"])
        elif include_empty:
            instance.last_login_time = 0
        if "userBan" in dict_ and dict_["userBan"] is not None:
            instance.user_ban = ModelSimpleUserBan.create_from_dict(
                dict_["userBan"], include_empty=include_empty
            )
        elif include_empty:
            instance.user_ban = ModelSimpleUserBan()
        if "wallets" in dict_ and dict_["wallets"] is not None:
            instance.wallets = [
                PlatformWalletAggResponse.create_from_dict(
                    i0, include_empty=include_empty
                )
                for i0 in dict_["wallets"]
            ]
        elif include_empty:
            instance.wallets = []
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelProgressionInfo]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelProgressionInfo]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ModelProgressionInfo,
        List[ModelProgressionInfo],
        Dict[Any, ModelProgressionInfo],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "createdAt": "created_at",
            "namespace": "namespace",
            "dlcs": "dlcs",
            "lastLoginTime": "last_login_time",
            "userBan": "user_ban",
            "wallets": "wallets",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "createdAt": True,
            "namespace": True,
            "dlcs": False,
            "lastLoginTime": False,
            "userBan": False,
            "wallets": False,
        }

    # endregion static methods
